#include <p32xxxx.h>
#include <stdlib.h>
#include <plib.h>
#include "Uart.h"
#include "HardwareProfile.h"
#include "BootLoader.h"
#include "Framework.h"

static UINT8 TxBuff[255];
BOOL GetChar(UINT8 *byte);
void PutChar(UINT8 txChar);

// Initializes UART.
void UartInit(UINT pbClk)
{
	// Open UART2 with Receive and Transmitter enable.
    UxBRG = (pbClk/16/DEFAULT_BAUDRATE-1); // calculate actual BAUD generate value.
    UxMODE = (UART_EN | UART_EN_CTS_RTS);
    UxSTA = (UART_RX_ENABLE | UART_TX_ENABLE);	
}	

// Closes UART connection.
void UartClose(void)
{
	//TODO: do some closing operation if required.	
}	

// Runs some UART routines like receive bytes and transmit bytes.
void UartTask(void)
{
	UINT8 TxLen;
	UINT8 Rx;
	UINT8 *ptr;
	// Check any character is received.
	if(GetChar(&Rx))
	{
		// Pass the bytes to frame work.
		FRAMEWORK_BuildRxFrame(&Rx, 1);
	}	
	
	ptr = TxBuff;
	// Get transmit frame from frame work.
	TxLen = FRAMEWORK_GetTransmitFrame(ptr);
	
	if(TxLen)
	{
		// There is something to transmit.
	   	while(TxLen--)
	   	{
		   PutChar(*(ptr++));
		} 
	} 
}

// Gets the data from UART RX FIFO.
BOOL GetChar(UINT8 *byte)
{
	BYTE dummy;

	if(UxSTA & 0x000E)              // receive errors?
	{
		dummy = UxRXREG; 			// dummy read to clear FERR/PERR
		UxSTAbits.OERR = 0;			// clear OERR to keep receiving
	}

	if(UxSTAbits.URXDA)
	{
		*byte = UxRXREG;		        // get data from UART RX FIFO
		return TRUE;
	}
	
	return FALSE;
}

// Puts the data into UART tx reg for transmission.
void PutChar(UINT8 txChar)
{
    while(UxSTAbits.UTXBF); // wait for TX buffer to be empty
    UxTXREG = txChar;
}
